#!/usr/bin/env node
/*!
 * gitmars v4.0.0-alpha.16
 * 这是一个git工作流工具
 * (c) 2021-2022 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import sh from 'shelljs';
import chalk from 'chalk';
import { queue } from '@gitmars/core/lib/queue';
import searchBranches from '@gitmars/core/lib/git/searchBranches';
import getIsGitProject from '@gitmars/core/lib/git/getIsGitProject';
import getIsBranchOrCommitExist from '@gitmars/core/lib/git/getIsBranchOrCommitExist';
import { createArgs } from '@gitmars/core/lib/utils/command';
import branchConfig from '#lib/conf/branch';
import i18n from '#lib/locales/index';

const { green, red } = chalk;
const { args, options } = branchConfig;
if (!getIsGitProject()) {
  sh.echo(
    red(i18n.__("The current directory is not a git project directory"))
  );
  process.exit(1);
}
program.name("gitm branch").usage(
  "[-k --key [keyword]] [-t --type [type]] [-d --delete [branch]] [--exclude [exclude]] [--include [include]] [-r --remote [remote]] [-D --forcedelete [branch]] [-u --upstream [upstream]]"
).description(
  i18n.__(
    "Branch query, delete (note that this command is not used to create a branch, if you want to create a branch please go through the start process)"
  )
);
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action((opt) => {
  const cmd = [];
  const isBranchExist = getIsBranchOrCommitExist(opt.delete);
  if (opt.delete) {
    if (isBranchExist)
      cmd.push(`git branch -d ${opt.delete}`);
    if (opt.remote)
      cmd.push(`git push origin --delete ${opt.delete}`);
    cmd.push({
      cmd: "git remote prune origin",
      config: {
        again: true,
        success: i18n.__("Cleanup of remote branch was successful"),
        fail: i18n.__(
          "Failed to clean up remote branch, please follow the prompts"
        )
      }
    });
  } else if (opt.forcedelete) {
    if (isBranchExist)
      cmd.push(`git branch -D ${opt.forcedelete}`);
    if (opt.remote)
      cmd.push(`git push origin --delete ${opt.delete}`);
    cmd.push({
      cmd: "git remote prune origin",
      config: {
        again: true,
        success: i18n.__("Cleanup of remote branch was successful"),
        fail: i18n.__(
          "Failed to clean up remote branch, please follow the prompts"
        )
      }
    });
  } else if (opt.upstream) {
    if (typeof opt.upstream === "string") {
      cmd.push(`git branch --set-upstream-to origin/${opt.upstream}`);
    } else {
      cmd.push("git branch --unset-upstream");
    }
  } else {
    const branches = searchBranches({
      remote: opt.remote,
      type: opt.type,
      key: opt.key,
      exclude: opt.exclude,
      include: opt.include
    });
    sh.echo(green(branches.join("\n")));
    return;
  }
  queue(cmd);
});
program.parse(process.argv);
