#!/usr/bin/env node
/*!
 * gitmars v4.0.0-alpha.16
 * 这是一个git工作流工具
 * (c) 2021-2022 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import sh from 'shelljs';
import chalk from 'chalk';
import { queue } from '@gitmars/core/lib/queue';
import getIsGitProject from '@gitmars/core/lib/git/getIsGitProject';
import getCurrentBranch from '@gitmars/core/lib/git/getCurrentBranch';
import checkGitStatus from '@gitmars/core/lib/git/checkGitStatus';
import { createArgs } from '@gitmars/core/lib/utils/command';
import copyConfig from '#lib/conf/copy';
import i18n from '#lib/locales/index';

const { red, yellow } = chalk;
const { args, options } = copyConfig;
if (!getIsGitProject()) {
  sh.echo(
    red(i18n.__("The current directory is not a git project directory"))
  );
  process.exit(1);
}
program.name("gitm copy").usage(
  "[commitid...] [-t --target [target]] [-k --key [keyword]] [-a --author [author]]"
).description(
  i18n.__(
    "cherry-pick batch version, copy a record from a branch and merge it into the current branch"
  )
);
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action((commitid, opts) => {
  const status = checkGitStatus();
  const cur = getCurrentBranch();
  if (!status)
    process.exit(1);
  if (commitid.length > 0) {
    const cmd = [
      {
        cmd: `git cherry-pick ${commitid.join(" ")}`,
        config: {
          again: false,
          success: i18n.__("Record merge successful"),
          fail: i18n.__(
            "Merge failed, please follow the instructions"
          )
        }
      }
    ];
    queue(cmd);
  } else if (!opts.key) {
    sh.echo(i18n.__("Please fill in the keyword"));
    process.exit(1);
  } else if (!opts.source) {
    sh.echo(i18n.__("Please fill in the source branch"));
    process.exit(1);
  } else {
    if (opts.key.length < 3) {
      sh.echo(
        yellow(
          i18n.__(
            "To make sure the copy is accurate, the keywords cannot be less than 4 words, please try to fill in the keywords completely"
          )
        )
      );
      process.exit(1);
    }
    const cmd = [
      `git checkout ${opts.source}`,
      `git log --grep=${opts.key} --author=${opts.author} --no-merges`
    ];
    queue(cmd).then((data) => {
      const commits = [];
      if (data[1].status === 0) {
        const logs = data[1].stdout.match(/(commit\s[a-z0-9]*\n+)/g) || [];
        let cmds = [`git checkout ${cur}`];
        logs.forEach((el) => {
          commits.push(el.replace(/(commit\s)|\n/g, ""));
        });
        commits.reverse();
        if (commits.length > 0) {
          cmds = cmds.concat([
            {
              cmd: `git cherry-pick ${commits.join(" ")}`,
              config: {
                again: false,
                success: i18n.__("Record merge successful"),
                fail: i18n.__(
                  "Merge failed, please follow the instructions"
                )
              }
            }
          ]);
        } else {
          sh.echo(i18n.__("No records found"));
        }
        queue(cmds);
      } else {
        sh.echo(data[1].stderr);
      }
    });
  }
});
program.parse(process.argv);
