#!/usr/bin/env node
/*!
 * gitmars v4.0.0-alpha.16
 * 这是一个git工作流工具
 * (c) 2021-2022 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import ora from 'ora';
import chalk from 'chalk';
import { createArgs } from '@gitmars/core/lib/utils/command';
import { spawnSync } from '@gitmars/core/lib/spawn';
import echo from '@gitmars/core/lib/utils/echo';
import installConfig from '#lib/conf/install';
import i18n from '#lib/locales/index';

const { green, red } = chalk;
const { args, options } = installConfig;
program.name("gitm install").usage(
  "<pluginName> [version] [-m --mirror] [-c --client [client]] [-r --registry <registry>]"
).description(i18n.__("Installing plugins, e.g. @gitmars/ui"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(
  (pluginName, version, opt) => {
    if (!pluginName) {
      echo(red(i18n.__("Please enter the plugin name")));
      process.exit(1);
    }
    const spinner = ora();
    if (version) {
      const match = version.match(/[0-9.]+$/);
      if (match)
        version = match[0];
      else if (![
        "alpha",
        "lite",
        "beta",
        "release",
        "latest",
        "next"
      ].includes(version)) {
        console.error(
          i18n.__(
            "Incorrect version number entered, only supported: alpha, lite, beta, release, latest, next"
          )
        );
        process.exit(0);
      }
    } else {
      version = "latest";
    }
    let cmdAdd;
    switch (opt.client) {
      case "yarn":
        cmdAdd = [
          opt.client,
          ["global", "add", `${pluginName}@${version}`]
        ];
        break;
      case "pnpm":
        cmdAdd = [opt.client, ["add", "-g", `${pluginName}@${version}`]];
        break;
      default:
        cmdAdd = [
          opt.client,
          ["install", "-g", `${pluginName}@${version}`]
        ];
        break;
    }
    if (!opt.registry && opt.mirror) {
      opt.registry = "https://registry.npmmirror.com";
    }
    if (opt.registry) {
      cmdAdd[1] = cmdAdd[1].concat(["-registry", opt.registry]);
    }
    spinner.start(green(i18n.__("Installing")));
    const install = spawnSync(...cmdAdd, {
      stdio: "ignore",
      shell: process.platform === "win32"
    });
    if (install.status === 0) {
      spinner.succeed(green(i18n.__("Installation complete")));
    } else {
      spinner.fail(
        red(
          i18n.__(
            "There was an installation error, please contact the administrator"
          )
        )
      );
    }
    spinner.stop();
    process.exit(0);
  }
);
program.parse(process.argv);
