#!/usr/bin/env node
/*!
 * gitmars v4.0.0-alpha.16
 * 这是一个git工作流工具
 * (c) 2021-2022 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import dayjs from 'dayjs';
import inquirer from 'inquirer';
import columnify from 'columnify';
import chalk from 'chalk';
import getUserToken from '@gitmars/core/lib/api/getUserToken';
import getIsGitProject from '@gitmars/core/lib/git/getIsGitProject';
import getGitConfig from '@gitmars/core/lib/git/getGitConfig';
import sendGroupMessage from '@gitmars/core/lib/sendGroupMessage';
import { createArgs } from '@gitmars/core/lib/utils/command';
import echo from '@gitmars/core/lib/utils/echo';
import getConfig from '@gitmars/core/lib/getConfig';
import { getMergeRequestList, getMergeRequestChanges, deleteMergeRequest, updateMergeRequest } from '@gitmars/core/lib/api/mergeRequest';
import { getMergeRequestNotesList, createMergeRequestNotes } from '@gitmars/core/lib/api/mergeRequestNotes';
import i18n from '#lib/locales/index';
import reviewConfig from '#lib/conf/review';

const { blue, cyan, green, magenta, red, yellow } = chalk;
const { args, options } = reviewConfig;
if (!getIsGitProject()) {
  echo(red(i18n.__("The current directory is not a git project directory")));
  process.exit(1);
}
const { appName } = getGitConfig();
const config = getConfig();
program.name("gitm review").usage("[--state [state]] [--quiet]").description(i18n.__("review remote code"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (opt) => {
  const userInfoApi = config.apis && config.apis.userInfo && config.apis.userInfo.url || config.api;
  const { token } = userInfoApi ? await getUserToken() : {};
  const mrList = await getMergeRequestList({ token, state: opt.state });
  if (mrList.length === 0) {
    echo(
      yellow(i18n.__("No merge request record found, process has exited"))
    );
    process.exit(0);
  }
  const prompt = [
    {
      type: "checkbox",
      message: i18n.__("Please select the merge request to be operated"),
      name: "iids",
      choices: []
    },
    {
      type: "list",
      message: i18n.__("Please select the action below."),
      name: "accept",
      choices: [
        i18n.__("View Details"),
        i18n.__("Comments"),
        i18n.__("Close"),
        i18n.__("Deleted"),
        i18n.__("Exit")
      ],
      when(answers) {
        return answers.iids.length;
      }
    }
  ];
  mrList.forEach((mr) => {
    const {
      iid,
      author,
      source_branch,
      target_branch,
      merge_status,
      created_at
    } = mr;
    const disabled = merge_status !== "can_be_merged";
    const _time = dayjs(created_at).format("YYYY/MM/DD HH:mm");
    prompt[0].choices.push({
      name: i18n.__(
        "{{id}} request merge {{{source}}} to {{{target}}} {{disabled}} | {{name}} | {{comments}} | {{time}}",
        {
          id: green(iid + ": "),
          source: green(source_branch),
          target: green(target_branch),
          disabled: disabled ? red(`[ ${i18n.__("Conflict or no need to merge")} ]`) : "",
          name: yellow(author.name),
          comments: green(
            i18n.__("{{length}} comments", {
              length: String(mr.notes.length)
            })
          ),
          time: blue(_time)
        }
      ),
      value: iid,
      checked: false
    });
  });
  const { iids, accept } = await inquirer.prompt(prompt);
  if (iids.length === 0) {
    echo(
      yellow(
        i18n.__("No merge request record selected, process has exited")
      )
    );
    process.exit(0);
  }
  for (const iid of iids) {
    const { source_branch, target_branch } = mrList.find(
      (item) => item.iid === iid
    );
    if (accept === i18n.__("View Details")) {
      const { changes, changes_count } = await getMergeRequestChanges({
        token,
        iid
      });
      echo(
        green(
          "\n" + i18n.__("{{id}}: total {{total}} files changed", {
            id: iid,
            total: changes_count
          })
        )
      );
      for (const { old_path, new_path, diff } of changes) {
        echo(
          magenta(
            "\n----------------------------------------------------------------------------------"
          )
        );
        echo(magenta(old_path));
        old_path !== new_path && echo(magenta(new_path + `(${i18n.__("New path")})`));
        echo(
          diff.replace(
            /(@@.+)\n/g,
            (m, p1) => cyan(p1) + "\n"
          ).replace(
            /\n(-.+)/g,
            (m, p1) => "\n" + red(p1)
          ).replace(
            /\n(\+.+)/g,
            (m, p1) => "\n" + green(p1)
          )
        );
      }
      const notes = (await getMergeRequestNotesList({
        token,
        iid
      }) || []).filter((note) => !note.system).map((note) => ({
        body: green(note.body),
        name: yellow(note.author.name),
        date: blue(
          dayjs(note.updated_at).format("YYYY/MM/DD HH:mm:ss")
        )
      }));
      echo(
        magenta(
          "\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
        )
      );
      echo(magenta(i18n.__("Comment List")));
      echo(
        columnify(notes, {
          columns: ["body", "name", "date"]
        })
      );
    } else if (accept === i18n.__("Deleted")) {
      await deleteMergeRequest({ token, iid });
      !opt.quiet && sendGroupMessage(
        i18n.__(
          "{{app}} item {{{source}}} merged to {{{target}}} request ID {{id}} has been deleted",
          {
            app: appName,
            source: source_branch,
            target: target_branch,
            id: iid
          }
        )
      );
      echo(green(i18n.__("Merge request {{id}}: Deleted", { id: iid })));
    } else if (accept === i18n.__("Close")) {
      await updateMergeRequest({
        token,
        iid,
        data: { state_event: "close" }
      });
      !opt.quiet && sendGroupMessage(
        i18n.__(
          "{{app}} item {{{source}}} merged to {{{target}}} request ID {{id}} has been closed",
          {
            app: appName,
            source: source_branch,
            target: target_branch,
            id: iid
          }
        )
      );
      echo(green(i18n.__("Merge request {{id}}: Closed", { id: iid })));
    } else if (accept === i18n.__("Comments")) {
      const { note } = await inquirer.prompt({
        type: "input",
        name: "note",
        message: i18n.__("Please enter the comment content"),
        default: "",
        transformer: (val) => val.trim(),
        validate: (val) => !val ? i18n.__("Please enter the available comments") : true
      });
      !opt.quiet && sendGroupMessage(
        i18n.__(
          "{{app}} item {{{source}}} merged to {{{target}}} request ID {{id}} has new comments: {{note}}",
          {
            app: appName,
            source: source_branch,
            target: target_branch,
            id: iid,
            note
          }
        )
      );
      await createMergeRequestNotes({ token, iid, body: note });
      echo(green(i18n.__("Submitted")));
    }
  }
});
program.parse(process.argv);
